#!/usr/bin/env python

import numpy as np
from matplotlib import animation
from matplotlib import pyplot as plt
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.widgets import Slider

from logslider import LogSlider, LogSlider0

π = np.pi

ω0 = 1.0
β = 0.1
x0 = 1.0
v0 = 0.0


def dampled_oscillations(tt, ω0, β, x0, v0):
    decay = np.exp(-β * tt)
    if β < ω0:
        ω = np.sqrt(ω0**2 - β**2)
        A = x0
        B = (v0 + β * x0) / ω
        xx = (A * np.cos(ω * tt) + B * np.sin(ω * tt)) * decay
        vv = ω * (-A * np.sin(ω * tt) + B * np.cos(ω * tt)) * decay - β * xx
        x1 = A * np.cos(ω * tt) * decay
        x2 = B * np.sin(ω * tt) * decay
        c1 = (A**2 + B**2)**0.5 * decay
        c2 = -c1
    elif β > ω0:
        ωi = np.sqrt(β**2 - ω0**2)
        A = x0
        B = (v0 + β * x0) / ωi
        xx = (A * np.cosh(ωi * tt) + B * np.sinh(ωi * tt)) * decay
        vv = ωi * (A * np.sinh(ωi * tt) + B * np.cosh(ωi * tt)) * decay - β * xx
        x1 = A * np.cosh(ωi * tt) * decay
        x2 = B * np.sinh(ωi * tt) * decay
        c1 = 0.5 * (A + B) * np.exp(-(β - ωi) * tt)
        c2 = 0.5 * (A - B) * np.exp(-(β + ωi) * tt)
    else:
        A = x0
        B = v0 + β * x0
        xx = (A + B * tt) * decay
        vv = B * decay - β * xx
        x1 = A * decay
        x2 = B * tt * decay
        c1 = c2 = np.nan * tt
    return xx, vv, x1, x2, c1, c2


# plt.style.use('dark_background')

fig, (ax1, ax2) = plt.subplots(nrows=2, sharex=True, gridspec_kw={'hspace': 0.1})

manager = fig.canvas.manager
manager.window.findChild(QtWidgets.QToolBar).setVisible(False)
manager.set_window_title("Drgania tłumione")
manager.full_screen_toggle()

ax1.axhline(0, color='0.3', lw=0.7)

a1 = ax1.axhline(1e1, color='0.8', lw=0.3)
a2 = ax1.axhline(-1e1, color='0.8', lw=0.3)
e1 = ax2.axhline(1e1, color='0.8', lw=0.3)
e2 = ax2.axhline(-1e1, color='0.8', lw=0.3)

tt = np.linspace(0., 10**1.75, 10001)

xx, vv, x1, x2, c1, c2 = dampled_oscillations(tt, ω0, β, x0, v0)
# line1, = ax.plot(tt, x1, lw=1.5, color='#81b1d2', ls='--')
# line2, = ax.plot(tt, x2, lw=1.5, color='#fa8174', ls='--')
# lineo, = ax.plot(tt, xx, lw=2.0, color='#feffb3')
linexx, = ax1.plot(tt, xx, lw=2.0, color='C0')
linex1, = ax1.plot(tt, x1, lw=0.5, color='C5')
linex2, = ax1.plot(tt, x2, lw=0.5, color='C2')
linec1, = ax1.plot(tt, c1, lw=0.5, color='C1', ls='--')
linec2, = ax1.plot(tt, c2, lw=0.5, color='C3', ls='--')

Ep = ω0**2 * xx**2
Ek = vv**2
E = Ep + Ek
lineep, = ax2.plot(tt, Ep, lw=1.0, color='C3', label="potencjalna")
lineek, = ax2.plot(tt, Ek, lw=1.0, color='C1', label="kinetyczna")
lineee, = ax2.plot(tt, E, lw=2.0, color='C0', label="całkowita")
lineea, = ax2.plot(tt, E[0] * np.exp(-2 * β * tt), '--', lw=0.5, color='0.5')
ax2.legend(loc='upper right')

ax1.set_ylim(-1., 1.)
ax1.set_yscale('symlog', linthresh=1e1)

ax2.set_xlabel('$t$ [s]')
ax2.set_xlim(tt[0], tt[-1])

ax2.set_ylabel("Energia")
ax2.set_ylim(0., 1.)
ax2.set_yscale('symlog', linthresh=1e1)

fig.tight_layout()

fig.subplots_adjust(bottom=0.22, left=0.07)
tax = fig.add_axes([0.25, 0.10, 0.35, 0.03])
ωax = fig.add_axes([0.25, 0.07, 0.65, 0.03])
βax = fig.add_axes([0.25, 0.04, 0.65, 0.03])
xax = fig.add_axes([0.25, 0.01, 0.30, 0.03])
vax = fig.add_axes([0.60, 0.01, 0.30, 0.03])
sax = fig.add_axes([0.70, 0.10, 0.20, 0.03])

sst = {
    'track_color': '0.7',
    'color': '0.7',
    'initcolor': 'black',
}

time_slider = LogSlider(ax=tax, label='Czas całkowity [s]', valmin=0.5, valmax=2, valinit=1.75, valstep=0.125, **sst)
scale_slider = Slider(ax=sax, label='Skala', valmin=-12, valmax=1, valinit=1, valstep=1, **sst)
omega_slider = Slider(ax=ωax, label='$\\omega_0$ [Hz]', valmin=0.0, valmax=2.0, valinit=ω0, valstep=0.01, **sst)
beta_slider = LogSlider0(ax=βax, label='$\\beta$ [Hz]', valmin=-2, valmax=1., valstep=0.01, valinit=np.log10(ω0), **sst)
x0_slider = Slider(ax=xax, label='$x_0$', valmin=0., valmax=2.0, valstep=0.1, valinit=x0, **sst)
v0_slider = Slider(ax=vax, label='$v_0$', valmin=-1.0, valmax=1.0, valstep=0.25, valinit=0., **sst)

beta_slider.set_val(np.log10(β))


def on_slider(val):
    global tt
    tt = np.arange(0., time_slider.val + 0.005, 0.01)
    ax1.set_xlim(tt[0], tt[-1])
    ω0 = omega_slider.val
    β = beta_slider.val
    x0 = x0_slider.val
    v0 = v0_slider.val
    xx, vv, x1, x2, c1, c2 = dampled_oscillations(tt, ω0, β, x0, v0)
    linex1.set_data(tt, x1)
    linex2.set_data(tt, x2)
    linexx.set_data(tt, xx)
    linec1.set_data(tt, c1)
    linec2.set_data(tt, c2)
    Ep = ω0**2 * xx**2
    Ek = vv**2
    E = Ep + Ek
    lineep.set_data(tt, Ep)
    lineek.set_data(tt, Ek)
    lineee.set_data(tt, E)
    lineea.set_data(tt, E[0] * np.exp(-2 * β * tt))
    lx = 10**scale_slider.val
    a1.set_ydata(lx)
    a2.set_ydata(-lx)
    e1.set_ydata(lx)
    e2.set_ydata(-lx)
    ax1.set_yscale('symlog', linthresh=lx)
    ax2.set_yscale('symlog', linthresh=lx)
    fig.canvas.draw_idle()


time_slider.on_changed(on_slider)
scale_slider.on_changed(on_slider)
omega_slider.on_changed(on_slider)
beta_slider.on_changed(on_slider)
x0_slider.on_changed(on_slider)
v0_slider.on_changed(on_slider)

plt.show()
